function _G.Serialize(obj, maxdepth)
    if (type(maxdepth) == "number") then
        maxdepth = maxdepth - 1;
    end
    if (type(obj) == "boolean") then
        if (obj) then
            return "true";
        else
            return "false";
        end
    elseif (type(obj) == "number") then
        local text = tostring(obj);
        -- Change floating-point numbers to English format
        return string.gsub(text, ",", ".");
    elseif (type(obj) == "string") then
        return string.format("%q", obj);
    elseif (type(obj) == "table") then
        if ((type(maxdepth) == "number") and (maxdepth < 0)) then
            return tostring(obj);
        else
            local text = "{";
            local i = 1;
            for key in sorted_keys(obj) do
                local value = Serialize(obj[key], maxdepth);
                if (value ~= nil) then
                    local item = value .. ",";
                    if (key ~= i) then
                        local index = Serialize(key, maxdepth);
                        item = "[" .. index .. "]=" .. item;
                    else
                        i = i + 1;
                    end
                    text = text .. item;
                end
            end
            text = string.gsub(text, ",$", "");
            text = text .. "}";
--text = tostring(obj) .. ":" .. text;
            return text;
        end
    else
--Turbine.Shell.WriteLine("unknown type " .. tostring(type));
        return tostring(obj);
    end
end

function _G.Bin2Text(data)
    data = data .. string.rep(string.char(0), 2); -- pad with null characters, if needed
    local string_byte, string_char = string.byte, string.char; -- local is faster than global
    local minChar = 46;
    local result, j, X1, X2, X3, Y1, Y2, Y3, Y4 = "", 1;
    for i = 1, #data - 2, 3 do
        X1, X2, X3 = string_byte(data, i, i + 2);
        Y1 = bit.brshift(X1, 2) + minChar;
        Y2 = bit.blshift(bit.band(X1, 3), 4) + bit.brshift(X2, 4) + minChar;
        Y3 = bit.blshift(bit.band(X2, 15), 2) + bit.brshift(X3, 6) + minChar;
        Y4 = bit.band(X3, 63) + minChar;
        result = result .. string_char(Y1, Y2, Y3, Y4);
        j = j + 4;
    end
    return result;
end

function _G.Text2Bin(data)
    local string_byte, string_char = string.byte, string.char; -- local is faster than global
    local minChar = 46;
    local result, j, X1, X2, X3, X4, Y1, Y2, Y3 = "", 1;
    for i = 1, #data - 3, 4 do
        X1, X2, X3, X4 = string_byte(data, i, i + 3);
        X1, X2, X3, X4 = X1 - minChar, X2 - minChar, X3 - minChar, X4 - minChar;
        Y1 = bit.blshift(X1, 2) + bit.brshift(X2, 4);
        Y2 = bit.blshift(bit.band(X2, 15), 4) + bit.brshift(X3, 2);
        Y3 = bit.blshift(bit.band(X3, 3), 6) + X4;
        result = result .. string_char(Y1, Y2, Y3);
        j = j + 3;
    end
    -- Remove up to two trailing null characters.
    return string.gsub(result, "%z$", "", 2);
end



Compresseur = Galphoglas.LotroChannFilter.LibCompress.LibCompress();

function export(index)
	lcfexport = Turbine.UI.Lotro.Window();
	lcfexport:SetSize(400,300);
	lcfexport:SetZOrder(2147483647);
	if (index==1) then
		lcfexport:SetPosition(lcfliste:GetLeft()+50,lcfliste:GetTop()+100);
	elseif (index==2) then
		lcfexport:SetPosition(lcfmodele:GetLeft()+50,lcfmodele:GetTop()+100);
	end
	lcfexport:SetText(lang["Exporter"][L]);
	lcfexport:SetVisible(true);
	
	lcfexport.Info = Turbine.UI.Label();
	lcfexport.Info:SetParent(lcfexport);
	lcfexport.Info:SetPosition(20,40);
	lcfexport.Info:SetSize(360,40);
	lcfexport.Info:SetFont(Turbine.UI.Lotro.Font.TrajanPro14);
	lcfexport.Info:SetForeColor(Turbine.UI.Color(0.8, 0.75, 0.5));
	lcfexport.Info:SetText(lang["ExporterInfo"][L]);

	lcfexport.Valeur = Turbine.UI.Lotro.TextBox();
	lcfexport.Valeur:SetParent(lcfexport);
	lcfexport.Valeur:SetPosition(20,90);
	lcfexport.Valeur:SetSize(360,190);
	lcfexport.Valeur:SetReadOnly(true);
	lcfexport.Valeur:SetForeColor(Turbine.UI.Color(1,1,1));
	lcfexport.Valeur:SetBackColor(Turbine.UI.Color(0, 0, 0));
	lcfexport.Valeur:SetMultiline(true);
	lcfexport.Valeur:SetVisible(true);
	lcfexport.Valeur:SetFont(Turbine.UI.Lotro.Font.TrajanPro14);
	
	compression(index);
	
end

function import(index)

	lcfimport = Turbine.UI.Lotro.Window();
	lcfimport:SetSize(400,300);
	lcfimport:SetZOrder(2147483647);
	if (index==1) then
		lcfimport:SetPosition(lcfliste:GetLeft()+50,lcfliste:GetTop()+100);
	elseif (index==2) then
		lcfimport:SetPosition(lcfmodele:GetLeft()+50,lcfmodele:GetTop()+100);
	end
	lcfimport:SetText(lang["Importer"][L]);
	lcfimport:SetVisible(true);
	
	lcfimport.Info = Turbine.UI.Label();
	lcfimport.Info:SetParent(lcfimport);
	lcfimport.Info:SetPosition(20,40);
	lcfimport.Info:SetSize(360,40);
	lcfimport.Info:SetFont(Turbine.UI.Lotro.Font.TrajanPro14);
	lcfimport.Info:SetForeColor(Turbine.UI.Color(0.8, 0.75, 0.5));
	lcfimport.Info:SetText(lang["ImporterInfo"][L]);

	lcfimport.Valeur = Turbine.UI.Lotro.TextBox();
	lcfimport.Valeur:SetParent(lcfimport);
	lcfimport.Valeur:SetPosition(20,90);
	lcfimport.Valeur:SetSize(360,190);
	lcfimport.Valeur:SetForeColor(Turbine.UI.Color(1,1,1));
	lcfimport.Valeur:SetBackColor(Turbine.UI.Color(0, 0, 0));
	lcfimport.Valeur:SetMultiline(true);
	lcfimport.Valeur:SetVisible(true);
	lcfimport.Valeur:SetFont(Turbine.UI.Lotro.Font.TrajanPro14);
	lcfimport.Valeur.TextChanged = function()
		importation(index);
	end
	
end

function compression(index)

	Valeur={};

	if (index==1) then
		for i=1,lcfliste.SaisieListe:GetItemCount()
		do
			temp=lcfliste.SaisieListe:GetItem(i);
			Valeur[i]=temp:GetText();
		end
		lcfexport.Valeur:SetText("{mot{"..Bin2Text(Compresseur:Compress(Serialize(Valeur))).."}/mot}");
	elseif (index==2) then
		for i=1,lcfmodele.SaisieListe:GetItemCount()
		do
			temp=lcfmodele.SaisieListe:GetItem(i);
			Valeur[i]=temp:GetText();
		end
		lcfexport.Valeur:SetText("{pat{"..Bin2Text(Compresseur:Compress(Serialize(Valeur))).."}/pat}");
	end
	lcfexport.Valeur:SelectAll();
	lcfexport.Valeur:Focus();
end

-- {mot{.`>e.>.\S;hQX\HX;/[Ce8?lXLPdC5CM@OXRm0@4>M_4GHNB57>PNhGBOc4Cg@Yb2<XWXZJXL6fXAGJXShDVQd.DjOc@UbP9_3jBI`D;RRWUWJA>@:g98EJSARTO<C:_M_Yfd[N1c^m3jd>^d/:L:VKP0j1Aj7.g3:lA^D/=^b=TB;/V;:]Iffi;=3a]`925YOk8KCScDBE88W?C88[MH/DEC@TY9X4VK@Vh/Gd0ASkWPUBX<YD4NTW]HT4_</>I.EPdPaNB4^5fB4^CN.m3CN3f/V[1Z>BBUZ5PB4^/b4L^<7Aae];L0j.aD:^^VGaeU]KH.1fB\kZ6j6A3VH.hX6f.=43_DN2.}/mot}
-- {mot{.`?M.>.\7KfYPIU.I0ECALV\h9G8\XlB177lRVFOlBeBB8DXNbXC>B]3FHEBJOCh@5P8L8`0gbXSh@W`D@SjO4HO]P=C7jBPjB\SeXVHWH9<8RC1Re7L@MS9g7@@;4JSEZUlk[W`R8mO]45MZ8l^Qi0=:7^ei/\4fiWH97H^83D5fiWH97G^?HRh51XK]\`Lg<=4\U^h`Z[/]VGa.^1.1>..CR>@QXd6O0[8cL4jFKl^Aj2l^Qi0=V5J3i0]Z4jFaUNh<OdJ1Rh515M2K5>hUU/V<e\IPc05Fc<6G1SA1ZLT22RfkPNP0JJL?@@@B6KQ8^f@@HQ1Z?C_P0ABfK^..5.1.6.CRBO09BUOd<KC?:705>i[>YZV6T4OaOF.}/mot}

function importation(index)

	voyage="";
	
	lcfimport:SetWantsUpdates(true);
	lcfimport.Cpt=0;
	lcfimport.Update=function()
	
		lcfimport.Cpt = lcfimport.Cpt + 1;
		
		if (lcfimport.Cpt == 1) then
			local temp = lcfimport.Valeur:GetText();
			if (index==1) then
				temp = temp:gsub("{mot{",""):gsub("}/mot}","");
			elseif (index==2) then
				temp = temp:gsub("{pat{",""):gsub("}/pat}","");
			end
			local ok, texte = pcall(Text2Bin, temp);
			if (not ok) then
				Turbine.Shell.WriteLine("<rgb=#FF0000>"..lang["ImporterErr"][L].."</rgb>");
				lcfimport.Cpt = 11;
			end
			voyage=texte;
		end
		if (lcfimport.Cpt == 2) then		
			local ok, texte = pcall(Compresseur.Decompress, Compresseur, voyage);
            if ((not ok) or (not texte)) then
				Turbine.Shell.WriteLine("<rgb=#FF0000>"..lang["ImporterErr"][L].."</rgb>");
				lcfimport.Cpt = 11;
			end
			recup=texte;
		end
		if (lcfimport.Cpt == 5) then
			recup=Disparition(recup:gsub("{\"",""):gsub("\"}","\",\""));			
			tableau={};
			j = 20
			for i=1, j
			do
				if string.find(recup, "\",\"") then
					tableau[i]=Apparition(string.sub(recup,1,string.find(recup, "\",\"")-1));
					recup=recup:gsub(Disparition(tableau[i]).."\",\"","");
				else
					break;
				end
			end
		end
		if (lcfimport.Cpt == 10) then

			Liste={};
			
			if (index==1) then
				for i=1,lcfliste.SaisieListe:GetItemCount()
				do
					temp=lcfliste.SaisieListe:GetItem(i);
					Liste[i]=temp:GetText();	
				end
			elseif (index==2) then
				for i=1,lcfmodele.SaisieListe:GetItemCount()
				do
					temp=lcfmodele.SaisieListe:GetItem(i);
					Liste[i]=temp:GetText();	
				end
			end
			
			k= table.getn(Liste);
			for i=1,table.getn(tableau)
			do
				local flag=1;
				for j=1,k
				do
					if (tableau[i]==Liste[j]) then
						flag=0;
					end			
				end
				if (flag==1) then
					table.insert(Liste,tableau[i]);
				else
					flag=1;
				end
			end
			
			if (index==1) then
				lcfliste.SaisieListe:ClearItems();
				for i=1,table.getn(Liste)
				do
					local item = Turbine.UI.Label();
					item:SetSize(390,14);
					item:SetFont(Turbine.UI.Lotro.Font.TrajanPro14);
					item:SetForeColor(Turbine.UI.Color(0,1,0));
					item:SetBackColor(Turbine.UI.Color(0, 0, 0));
					item:SetText(Liste[i]);
					lcfliste.SaisieListe:AddItem(item);
				end
			elseif (index==2) then
				lcfmodele.SaisieListe:ClearItems();
				for i=1,table.getn(Liste)
				do
					local item = Turbine.UI.Label();
					item:SetSize(390,14);
					item:SetFont(Turbine.UI.Lotro.Font.TrajanPro14);
					item:SetForeColor(Turbine.UI.Color(0,1,0));
					item:SetBackColor(Turbine.UI.Color(0, 0, 0));
					item:SetText(Liste[i]);
					lcfmodele.SaisieListe:AddItem(item);
				end
			end
		end
		if (lcfimport.Cpt == 11) then
			lcfimport:SetWantsUpdates(false);
			if (index==1) then
				saveMots();
			elseif (index==2) then
				saveModele();
			end
			lcfimport:Close();
		end
	end	
end

function detection()
	
end